#include <string>
#include <thread>
#include <gst/gst.h>
#include <iostream>

void
start_pipeline(GstElement * pipeline) {
    GstBus *bus;
    GstMessage *msg;

    /* Start playing */
    gst_element_set_state (pipeline, GST_STATE_PLAYING);

    /* Wait until error or EOS */
    bus = gst_element_get_bus (pipeline);
    msg = gst_bus_timed_pop_filtered (bus, GST_CLOCK_TIME_NONE, static_cast<GstMessageType>(GST_MESSAGE_ERROR | GST_MESSAGE_EOS));

    /* Free resources */
    if (msg != NULL)
        gst_message_unref (msg);
    gst_object_unref (bus);
    gst_element_set_state (pipeline, GST_STATE_NULL);
    gst_object_unref (pipeline);
}


int
main (int argc, char *argv[])
{
    /* Initialize GStreamer */
    gst_init (&argc, &argv);

    const gchar * pipeline_desc_0 =
        "nvarguscamerasrc sensor-id=0"
        " ! nvivafilter customer-lib-name=../gst-nvivafilter/librotation_filter.so cuda-process=true"
        " ! video/x-raw(memory:NVMM),width=1280,height=720,format=NV12"
        " ! nv3dsink";
    
    const gchar * pipeline_desc_1 =
        "nvarguscamerasrc sensor-id=1"
        " ! nvivafilter customer-lib-name=../gst-nvivafilter/librotation_filter.so cuda-process=true"
        " ! video/x-raw(memory:NVMM),width=1280,height=720,format=NV12"
        " ! nv3dsink";

    /* Build two pipelines */
    GstElement * pipeline0;
    pipeline0 = gst_parse_launch(pipeline_desc_0, NULL);
    GstElement * pipeline1;
    pipeline1 = gst_parse_launch(pipeline_desc_1, NULL);
    
    std::thread thread0(start_pipeline, pipeline0);
    std::thread thread1(start_pipeline, pipeline1);

    thread0.join();
    thread1.join();
    
    return 0;
}